/*
 * Copyright (C) 2013 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Juha Ristolainen <juha.ristolainen@codemancers.fi>
 */

import QtQuick 2.0
import Ubuntu.Components 0.1
import Ubuntu.Components.ListItems 0.1
import "../common/ClockUtils.js" as Utils

/*!
    \brief AlarmPage is a page that lives inside the "Alarm"-tab of the application
*/

Page {
    id: alarmPageRoot

    ////////////////////////////////////////////////////////////////////////////////
    // Properties of this page
    anchors.margins: units.gu(2)

    ////////////////////////////////////////////////////////////////////////////////
    // New properties of this page
    property var alarmTime: new Date();
    property string alarmTimeString

    ////////////////////////////////////////////////////////////////////////////////
    // Property changed handlers
    onAlarmTimeChanged: {
        alarmTimeString = Qt.formatTime(alarmTime)
    }

    ////////////////////////////////////////////////////////////////////////////////
    // Initialization after page has loaded
    Component.onCompleted: {
        Utils.log("AlarmPage loaded");
    }

    ////////////////////////////////////////////////////////////////////////////////
    // Main Timer-callback from clock.qml
    function onTimerUpdate(now) {
    }

    ////////////////////////////////////////////////////////////////////////////////
    // Helper-functions

    function createAlarm(label, time) {
        Utils.log("createAlarm: " + label + " for " + time);
        return { label: label, time: time, active: "false" };
    }

    ////////////////////////////////////////////////////////////////////////////////
    // Non-UI elements
    ListModel {
        id: alarmsModel
    }


    ////////////////////////////////////////////////////////////////////////////////
    // UI
    Column {
        width: parent.width
        height: parent.height
        spacing: units.gu(1)

        Label {
            id: alarmTimeLabel
            width: parent.width
            horizontalAlignment: Text.AlignHCenter
            verticalAlignment: Text.AlignVCenter
            fontSize: "x-large"
            text: alarmPageRoot.alarmTimeString
        }

        // TODO: create a nice time picker
        Rectangle {
            id: timeChooser
            width: parent.width
            height: parent.width / 3
        }

        /*
        Button {
            id: clearAlarmsButton
            width: parent.width
            text: i18n.tr("Clear all")
            onClicked: clearAlarms();
        }

        Button {
            id: addAlarmButton
            width: parent.width
            text: i18n.tr("Add alarm")
            onClicked: addAlarm();
        }*/

        ListView {
            id: alarmsList
            width: parent.width
            height: parent.height - alarmTimeLabel.height - timeChooser.height
            model: alarmsModel
            delegate: alarmDelegate
        }
    }

    Component {
        id: alarmDelegate
        Subtitled {
            text: label
            subText: Qt.formatDateTime(time)
            onClicked: removeAlarm(label);
        }
    }

    /*
    tools: ToolbarActions {
        Action {
            objectName: "action"

            iconSource: Qt.resolvedUrl("../images/add.png")
            text: i18n.tr("Add alarm")
        }
    }
    */
}
