/*
 * Copyright (C) 2013 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Juha Ristolainen <juha.ristolainen@codemancers.fi>
 *              Nekhelesh Ramananthan <krnekhelesh@gmail.com>
 *              Nick Leppänen Larsson <frals@frals.se>
 */

import QtQuick 2.0
import Ubuntu.Components 0.1
import "../common/ClockUtils.js" as Utils

/*!
    \brief ClockPage is a page that lives inside the "Clock"-tab of the application
*/

Page {
    Component.onCompleted: {
        Utils.log("ClockPage loaded");
        currentTimeFormatted = Qt.formatTime(new Date());
        selectedLocation = "USER'S LOCATION TODO"
    }

    // Currently loaded clock face filename
    // TODO better enumeration to filename mapping
    property string currentClockFaceFilename: "../common/AnalogClockFace.qml"

    // Property to hold the formatted time string to show on the screen
    property string currentTimeFormatted

    // Property to hold the selected location to show time for
    // TODO: how to get user's set location?
    property string selectedLocation

    /*!
      \brief main Timer calls this on every 1000ms
      */
    function onTimerUpdate(now) {
        currentTimeFormatted = Qt.formatTime(now);

        // Tell currently loaded clockface that time has changed
        clockFaceLoader.item.timeChanged(now);
    }

    Label {
        id: currentTimeLabel
        width: parent.width
        height: parent.height / 4
        horizontalAlignment: Text.AlignHCenter
        verticalAlignment: Text.AlignVCenter
        fontSize: "x-large"
        text: currentTimeFormatted
    }

    // TODO: load from settings which clock face to display
    // TODO: create digital clock face
    Loader {
        id: clockFaceLoader
        anchors.centerIn: parent
        source: currentClockFaceFilename

        Component.onCompleted:  {
            Utils.log("Clock face loaded from " + source);
        }
    }

    Label {
        id: locationLabel
        width: parent.width
        height: parent.height / 4

        anchors.bottom: parent.bottom

        horizontalAlignment: Text.AlignHCenter
        verticalAlignment: Text.AlignVCenter
        fontSize: "large"
        text: selectedLocation
    }

}
