/*
 * Copyright (C) 2013 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Juha Ristolainen <juha.ristolainen@codemancers.fi>
 *              Nekhelesh Ramananthan <krnekhelesh@gmail.com>
 *              Nick Leppänen Larsson <frals@frals.se>
 */

import QtQuick 2.0
import Ubuntu.Components 0.1
import "../common/Constants.js" as Constants

AnalogFaceBase {
    id: clockRoot

    function timeChanged(now) {
        hours = now.getHours()
        minutes = now.getMinutes()
        seconds = now.getUTCSeconds();
    }

    property var currDate: new Date;
    property int hours: currDate.getHours()
    property int minutes: currDate.getMinutes()
    property int seconds: currDate.getUTCSeconds();

    // "API" of the analog face
    // say you want add a mousearea to one of the hands
    // (e.g. when doing a timepicker), just do
    //
    /*
    minuteHandRect.children: MouseArea {
        anchors.fill: parent
        onClicked: // do something
    }

    */
    property alias hourHandRect: hourHand
    property alias minuteHandRect: minuteHand
    property alias secondsHandRect: secondsHand

    pulseGlow: false

    AnalogClockHand {
        id: hourHand
        height: units.gu(8)
        // TODO: set source according to theme?
        rotationAngle: (clockRoot.hours * 30) + (clockRoot.minutes / 2)
    }

    AnalogClockHand {
        id: minuteHand
        height: units.gu(12)
        // TODO: set source according to theme?
        rotationAngle: clockRoot.minutes * 6
    }

    AnalogClockHand {
        id: secondsHand
        // TODO: set source according to theme?
        height: units.gu(14)
        width:  units.gu(0.5)
        rotationAngle: clockRoot.seconds * 6
    }

    Repeater {
        model: ["1", "2", "4", "5", "7", "8", "10", "11"]
        delegate: AnalogClockMarker {
            number: modelData
            size:0.7
        }
    }

    Repeater {
        model: ["3", "6", "9", "12"]
        delegate: AnalogClockMarker {
            number: modelData
            size: 1
            color: Constants.pitchBlack
        }
    }
}
