/*
 * Copyright (C) 2013 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Juha Ristolainen <juha.ristolainen@codemancers.fi>
 *              Nick Leppänen Larsson <frals@frals.se>
 */

import QtQuick 2.0
import Ubuntu.Components 0.1
import "../common/Constants.js" as Constants

Rectangle {
    id: clockRoot

    // set this to false when app goes to background,
    // it should stop most of the active animations
    property bool isForeground: true
    // deactivate pulsing of the glow effect
    property bool pulseGlow: isForeground
    property color glowColor: Constants.analogClockFaceGlowColor

    anchors.centerIn: parent
    radius: width / 2
    antialiasing: true

    width: units.gu(34)
    height: units.gu(34)

    Rectangle {
        id: innerCircle
        anchors.centerIn: parent
        z: 1
        width: units.gu(2)
        height: units.gu(2)
        color: Constants.pitchBlack
        radius: width / 2
        antialiasing: true
    }

    AnalogGlowEffect {
        id: glowEffect
        z: parent.z - 1
        fillRectangle: parent
        /* this animation might be a bit much considering
          that the seconds hand is already moving...
          but i like it :P */
        SequentialAnimation on opacity {
            running: clockRoot.pulseGlow
            loops: Animation.Infinite
            NumberAnimation {
                target: glowEffect
                property: "opacity"
                to: 1.0
                duration: 500
            }
            NumberAnimation {
                target: glowEffect
                property: "opacity"
                to: 0.5
                duration: 500
            }
        }
    }

}
