/*
 * Copyright (C) 2013 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Juha Ristolainen <juha.ristolainen@codemancers.fi>
 */

import QtQuick 2.0
import QtQuick.LocalStorage 2.0
import "../common/ClockUtils.js" as Utils

QtObject {
    id: storageRoot

    property var db: null

    Component.onCompleted: {
        Utils.log("Settings loaded")
        openDB();
    }

    // Opens the local db if not already open. On the first call creates the tables
    function openDB() {
        if(db !== null) return;

        db = LocalStorage.openDatabaseSync("ubuntu-clock-app", "0.1", "Default Ubuntu touch clock", 1000);

        try {
            db.transaction(function(tx){
                tx.executeSql('CREATE TABLE IF NOT EXISTS Settings(key TEXT, value TEXT)');
            });
        } catch (err) {
            Utils.error("Error creating table in db");
        }
    }

    function getSettingsValue(key, defaultValue) {
        openDB();

        var result = defaultValue;
        try {
            db.readTransaction(
                        function(tx){
                            var res = tx.executeSql('SELECT value FROM Settings WHERE key=?', key);
                            if (res.rows.length > 0) {
                                var item = res.rows.item(0);
                                result = item.value;
                            }
                        });
        } catch (err) {
            Utils.error("Error getting setting '" + key + "': " + err);
        }
        return result;
    }

    function setSettingsValue(key, value) {
        openDB();

        try {
            db.transaction(function(tx){
                var currentValue;
                var res = tx.executeSql('SELECT value FROM Settings WHERE key=?', key);
                if (res.rows.length > 0) {
                    currentValue = res.rows.item(0).value;
                }

                if (currentValue !== value) {
                    // Update existing value or insert if none
                    if (currentValue !== undefined) {
                        res = tx.executeSql('UPDATE Settings SET value = ? WHERE key = ?', [value, key]);
                    } else {
                        res = tx.executeSql('INSERT INTO Settings VALUES(?, ?)', [key, value]);
                    }
                }
            });
        } catch (err) {
            Utils.error("Error setting key '"+ key + "': " + err);
        }
    }
}
