/*
 * Copyright (C) 2013 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Alessandro Pozzi <signor.hyde@gmail.com>
 */

import QtQuick 2.0
import Ubuntu.Components 0.1
import Ubuntu.Components.Popups 0.1
import Ubuntu.Components.ListItems 0.1 as ListItem
import "../common/Constants.js" as Constants
import "StopwatchScript.js" as SWScript
import "../common"

Page {

    property real minutes: 0;
    property real seconds: 0;
    property real decimals: 0;
    property real minuteHandValue: minutes
    property real secondHandValue: seconds
    property real perRadius: 6
    property string timelap
    property int countlap


    Component.onCompleted: {
        SWScript.log("StopwatchPage loaded");
    }

    function onTimerUpdate() {
    }

    Timer {
        id: stopwatchTimer
        interval: 100; running: false; repeat: true;
        onTriggered: {
            SWScript.increaseDecimal()
        }
    }

    anchors.margins: units.gu(2)

    Label {
        id: labelStopwatch

        width: parent.width
        height: parent.height / 4

        horizontalAlignment: Text.AlignHCenter
        verticalAlignment: Text.AlignVCenter

        fontSize: "x-large"
        text: i18n.tr("00:00:00")
    }


    AnalogFaceBase {
        id: clockOuterCircle

        Repeater {
            id:repeaterMarket

            model: ["1", "2", "4", "5", "7", "8", "10", "11"]

            delegate: AnalogClockMarker {
                number: modelData
                size: 0.5
                distance: 0.45
                color: Constants.pitchBlack
            }
        }

        MouseArea {
            anchors.fill: parent

            onClicked: {
                if (circleMinutes.state == "NORMAL") {
                    circleMinutes.state = "EGG";
                } else {
                    circleMinutes.state = "NORMAL";
                }
            }
        }
    }

    Label {
        id: point60

        anchors {
            horizontalCenter: clockOuterCircle.horizontalCenter
            top: clockOuterCircle.top
            topMargin: 6
        }

        fontSize: "medium"
        text: i18n.tr("60")
    }

    Label {
        id: point15

        anchors {
            verticalCenter: clockOuterCircle.verticalCenter
            right: clockOuterCircle.right
            rightMargin: 6
        }

        fontSize: "medium"
        text: i18n.tr("15")
    }

    Label {
        id: point30

        anchors {
            horizontalCenter: clockOuterCircle.horizontalCenter
            bottom: clockOuterCircle.bottom
            bottomMargin: 6
        }

        fontSize: "medium"
        text: i18n.tr("30")
    }

    Label {
        id: point45

        anchors {
            verticalCenter: clockOuterCircle.verticalCenter
            left: clockOuterCircle.left
            leftMargin: 6
        }

        fontSize: "medium"
        text: i18n.tr("45")
    }

    AnalogHand {
        id: secondHand

        rotationAngle: parseInt(stopwatchPage.secondHandValue) * perRadius

        colorhand: Constants.brightWhite
        colortop: Constants.ubuntuOrange

        widthhand: units.gu(0.5)
        heighthand: units.gu(17)
        heighttop: units.gu(5)
    }

    Rectangle {
        id: circleMinutes

        width: units.gu(15)
        height: units.gu(15)

        anchors.centerIn: parent

        color: Constants.normalGrey
        radius: width / 2
        antialiasing: true

        states:[
            State {
                name: "NORMAL"

                PropertyChanges {
                    target: point60;
                    text: i18n.tr("60")
                }

                PropertyChanges {
                    target: point15;
                    text: i18n.tr("15")
                }

                PropertyChanges {
                    target: point30;
                    text: i18n.tr("30")
                }

                PropertyChanges {
                    target: point45;
                    text: i18n.tr("45")
                }

                PropertyChanges {
                    target: minutsPoint60;
                    visible: false;
                }

                PropertyChanges {
                    target: stopwatchPage;
                    minuteHandValue: minutes;
                }

                PropertyChanges {
                    target: stopwatchPage;
                    secondHandValue: seconds;
                }

                PropertyChanges {
                    target: stopwatchPage;
                    perRadius: 6;
                }

                PropertyChanges {
                    target: repeaterMarket;
                    model: ["1", "2", "4", "5", "7", "8", "10", "11"];
                }
            },

            State {
                name: "EGG"

                PropertyChanges {
                    target: point60;
                    text: i18n.tr("100")
                }

                PropertyChanges {
                    target: point15;
                    text: i18n.tr("25")
                }

                PropertyChanges {
                    target: point30;
                    text: i18n.tr("50")
                }

                PropertyChanges {
                    target: point45;
                    text: i18n.tr("75")
                }

                PropertyChanges {
                    target: minutsPoint60;
                    visible: true;
                }

                PropertyChanges {
                    target: stopwatchPage;
                    minuteHandValue: seconds;
                }

                PropertyChanges {
                    target: stopwatchPage;
                    secondHandValue: decimals;
                }

                PropertyChanges {
                    target: stopwatchPage;
                    perRadius: 36;
                }

                PropertyChanges {
                    target: repeaterMarket;
                    model: [];
                }
            }
        ]

        MouseArea {
            anchors.fill: parent

            onClicked: {
                if (parent.state == "NORMAL") {
                    parent.state = "EGG";
                } else {
                    parent.state = "NORMAL";
                }
            }
        }
    }

    Label {
        id: minutsPoint60

        anchors {
            horizontalCenter: circleMinutes.horizontalCenter
            top: circleMinutes.top
            topMargin: 6
        }

        visible: false

        fontSize: "medium"
        text: i18n.tr("60")
    }

    AnalogHand {
        id: minuteHand

        rotationAngle: parseInt(stopwatchPage.minuteHandValue) * 6
        colorhand: Constants.pitchBlack
        colortop: Constants.ubuntuOrange

        widthhand: units.gu(0.5)
        heighthand: units.gu(6)
        heighttop: units.gu(0)
    }

    Row {
        spacing: units.gu(2)

        anchors {
            horizontalCenter: parent.horizontalCenter
            bottom: parent.bottom
            bottomMargin: units.gu(1)
        }

        Button {
            id: buttonStopwatch

            width: units.gu(12)
            height: units.gu(5)


            color: Constants.green
            text: i18n.tr("Start")

            onClicked: {
                if(stopwatchTimer.running == false) {
                    stopwatchTimer.start();
                    text = i18n.tr("Stop");
                    color = Constants.ubuntuOrange;
                    buttonReset.text = i18n.tr("Lap");
                    buttonReset.color = Constants.brightWhite;
                } else {
                    stopwatchTimer.stop();
                    text = i18n.tr("Start");
                    color = Constants.green;
                    buttonReset.text = i18n.tr("Reset");
                    buttonReset.color = Constants.normalGrey;
                }
            }
        }

        UbuntuShape {
            id: shapeLap

            clip: true

            width: units.gu(7)
            height: units.gu(7)

            anchors.verticalCenter: buttonStopwatch.verticalCenter

            color: Constants.brightWhite

            Label {
                id: labelNumberLap1

                width: parent.width
                height: parent.height

                horizontalAlignment: Text.AlignHCenter
                verticalAlignment: Text.AlignVCenter

                color: Constants.ubuntuOrange
                fontSize: "x-large"
                text: i18n.tr("0")

                PropertyAnimation {
                    id: move
                    target: labelNumberLap1
                    properties: "y"
                    from: units.gu(0)
                    to: - units.gu(6)
                    duration: 750
                    running: false
                }

                MouseArea {
                    anchors.fill: parent

                    onClicked: {
                        PopupUtils.open(listLap, labelNumberLap1)
                    }
                }
            }

            Label {
                id: labelNumberLap2

                width: parent.width
                height: parent.height

                y: parent.y + parent.width

                horizontalAlignment: Text.AlignHCenter
                verticalAlignment: Text.AlignVCenter

                color: Constants.ubuntuOrange
                fontSize: "x-large"
                text: i18n.tr(countlap + 1)

                PropertyAnimation {
                    id: move2
                    target: labelNumberLap2
                    properties: "y"
                    from: units.gu(6)
                    to: units.gu(0)
                    duration: 750
                    running: false
                }

                MouseArea {
                    anchors.fill: parent
                    onClicked: {
                        PopupUtils.open(listLap, labelNumberLap2)
                    }
                }
            }
        }

        Button {
            id: buttonReset

            width: units.gu(12)
            height: units.gu(5)

            text: i18n.tr("Reset")
            color: Constants.normalGrey

            onClicked: {
                if(text == "Reset") {
                    SWScript.reset();
                } else {
                    SWScript.lap();
                    labelNumberLap2.text = countlap
                    labelNumberLap1.text = countlap - 1
                    move.start()
                    move2.start()
                }
            }
        }
    }

    Component {
        id: listLap

        Popover {
            id: popupLap

            ListView {
                id: labelCurrentLab

                clip: true

                anchors {
                    left: parent.left
                    right: parent.right
                }

                height: units.gu(60)

                ListItem.Header {
                    id: lapHeader
                    text: i18n.tr("Laps")
                }

                ListItem.SingleValue {
                    id: currentLapItem

                    anchors.top: lapHeader.bottom

                    text: i18n.tr("Current")
                    value: timelap
                }

                ListItem.Header {
                    id: currentLapHeader

                    anchors.top: currentLapItem.bottom

                    text: i18n.tr("Recorded laps")
                }

                ListView {
                    clip: true

                    anchors {
                        left: parent.left
                        right: parent.right
                        top: currentLapHeader.bottom
                    }

                    model: SWScript.laplist()

                    height: units.gu(45)

                    delegate: ListItem.SingleValue {
                        text: SWScript.laplist().length - index

                        value: modelData

                        onClicked: {
                            PopupUtils.close(popupLap)
                        }
                    }
                }
            }
        }
    }
}
