/*
 * Copyright (C) 2013 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Alessandro Pozzi <signor.hyde@gmail.com>
 *              Nekhelesh Ramananthan <krnekhelesh@gmail.com>               
 */

import QtQuick 2.0
import Ubuntu.Components 0.1
import "../common"
import "../common/Constants.js" as Constants
import "../common/ClockUtils.js" as Utils
import "TimerScript.js" as TScript

Page {
    anchors.margins: units.gu(2)

    property bool timerOn: false

    property int seconds: 0;
    property int minutes: 0;
    property int hours: 0;
    property int totalTime: 0;
    property int timerValue: 0;

    Component.onCompleted: {
        Utils.log("TimerPage loaded");
    }

    // Timer function called by the main clock loop
    function onTimerUpdate() {
        if(timerOn == true) {
            TScript.countDown()
            timerHand.rotationAngle = seconds * 6;
        }
    }

    // Timer label hh:mm:ss
    Row {

        spacing: units.gu(0.3)

        anchors {
            horizontalCenter: parent.horizontalCenter
            top: parent.top
            topMargin: units.gu(6)
        }

        Label {
            id: hourslabelTimer

            // flag boolean variable to determine if the timer hand adjusts the hours or not
            property bool flag: false

            color: Constants.normalGrey

            fontSize: "x-large"
            text: i18n.tr("00")
            visible: {
                if (hours > 0) {
                    true
                }
                else {
                    true
                    /* FIXME: According to design the hour label should only be shown when the
                      minutes hand is moved beyond 59 minutes. The logic for this has not been
                      implemented yet in this code.
                      */
                    // TODO: Change this to false later. As of now the hour label is always shown.
                }
            }

            MouseArea {
                anchors.fill: parent

                onClicked: {
                    timerHand.rotationAngle = hours * 6;
                    hourslabelTimer.color = Constants.ubuntuOrange;
                    hourslabelTimer.flag = true;
                    secondslabelTimer.color = Constants.normalGrey;
                    secondslabelTimer.flag = false;
                    minuteslabelTimer.color = Constants.normalGrey;
                    minuteslabelTimer.flag = false;
                }
            }
        }

        Label {
            id: colonTimer1

            color: Constants.normalGrey
            visible: {
                if (hourslabelTimer.visible) {
                    true
                }
                else {
                    false
                }
            }

            fontSize: "x-large"
            text: i18n.tr(":")
        }

        Label {
            id: minuteslabelTimer

            // flag boolean variable to determine if the timer hand adjusts the minutes or not
            property bool flag: false

            color: Constants.normalGrey

            fontSize: "x-large"
            text: i18n.tr("00")

            MouseArea {
                anchors.fill: parent

                onClicked: {
                    timerHand.rotationAngle = minutes * 6;
                    minuteslabelTimer.color = Constants.ubuntuOrange;
                    minuteslabelTimer.flag = true;
                    secondslabelTimer.color = Constants.normalGrey;
                    secondslabelTimer.flag = false;
                    hourslabelTimer.color = Constants.normalGrey;
                    hourslabelTimer.flag = false;
                }
            }
        }

        Label {
            id: colonTimer2

            color: Constants.normalGrey

            fontSize: "x-large"
            text: i18n.tr(":")
        }

        Label {
            id: secondslabelTimer

            // flag boolean variable to determine if the timer hand adjusts the minutes or not
            property bool flag: false

            color: Constants.normalGrey

            fontSize: "x-large"
            text: i18n.tr("00")

            MouseArea {
                anchors.fill: parent

                onClicked: {
                    timerHand.rotationAngle = seconds * 6;
                    secondslabelTimer.color = Constants.ubuntuOrange;
                    secondslabelTimer.flag = true;
                    minuteslabelTimer.color = Constants.normalGrey;
                    minuteslabelTimer.flag = false;
                    hourslabelTimer.color = Constants.normalGrey;
                    hourslabelTimer.flag = false;

                }
            }
        }

    }

    AnalogFaceBase {
        id: clockOuterCircle

        // Implements touch support
        MouseArea {
            id: timerbackmousearea

            property real truex: mouseX - clockOuterCircle.width/2
            property real truey: clockOuterCircle.height/2 - mouseY
            property real angle: Math.atan2(truex, truey)
            property real strictangle: parseInt(angle * 180 / Math.PI)
            property real modulo: strictangle % 6

            /* Touch support to move timer hand should only be enabled when the timer is off,
              and the user has clicked on any of the second, minute and hour label.
              */
            enabled: {
                if (timerOn) {
                    false
                }
                else if (secondslabelTimer.flag) {
                    true
                }
                else if (minuteslabelTimer.flag) {
                    true
                }
                else if (hourslabelTimer.flag) {
                    true
                }
                else {
                    false
                }
            }

            anchors.fill: parent
            preventStealing: true // move only Timer hand and ensures that the sideway motion of pages do not interrupt moving the clock hand
            onPositionChanged:  if (buttonTimer.state != "STOP" && buttonTimer.state != "DONE") {
                                    if (timerbackmousearea.angle < 0) {
                                        timerHand.rotationAngle = strictangle - modulo + 360;
                                        timerValue = parseInt(timerHand.rotationAngle/6);

                                        if (secondslabelTimer.flag) {
                                            secondslabelTimer.text = Utils.zeroleft(timerValue, 2);
                                            seconds = timerValue;
                                        }

                                        if (minuteslabelTimer.flag) {
                                            minuteslabelTimer.text = Utils.zeroleft(timerValue, 2);
                                            minutes = timerValue;
                                        }

                                        if (hourslabelTimer.flag) {
                                            hourslabelTimer.text = Utils.zeroleft(timerValue, 2);
                                            hours = timerValue;
                                        }

                                        if(secondslabelTimer.text != "00" || minuteslabelTimer.text != "00" || hourslabelTimer.text != "00") {
                                            buttonTimer.state = "START";
                                        } else {
                                            buttonTimer.state = "PRESETS";
                                        }
                                    }
                                    else {
                                        timerHand.rotationAngle = strictangle - modulo + 6;
                                        timerValue = parseInt(timerHand.rotationAngle/6);

                                        if (secondslabelTimer.flag) {
                                            secondslabelTimer.text = Utils.zeroleft(timerValue, 2);
                                            seconds = timerValue;
                                        }

                                        if (minuteslabelTimer.flag) {
                                            minuteslabelTimer.text = Utils.zeroleft(timerValue, 2);
                                            minutes = timerValue;
                                        }

                                        if (hourslabelTimer.flag) {
                                            hourslabelTimer.text = Utils.zeroleft(timerValue, 2);
                                            hours = timerValue;
                                        }

                                        if(secondslabelTimer.text != "00" || minuteslabelTimer.text != "00" || hourslabelTimer.text != "00") {
                                            buttonTimer.state = "START";
                                        } else {
                                            buttonTimer.state = "PRESETS";
                                        }
                                    }
                                }
        }

        Repeater {
            model: 12
            delegate: AnalogClockMarker {
                number: index
                size: 0.5
                distance: 0.55
                color: Constants.pitchBlack
            }
        }
    }

    AnalogHand {
        id: timerHand

        widthhand: units.gu(2)
        heighthand: units.gu(16)
        heighttop: units.gu(5)

        rotationAngle: seconds * 6

        colorhand: Constants.brightWhite
        colortop: Constants.ubuntuOrange
    }

    Rectangle {
        id: secondsCircle

        width: units.gu(5)
        height: units.gu(5)

        anchors.centerIn: parent

        color: Constants.brightWhite
        radius: width / 2
        antialiasing: true
    }

    // Add preset done circle which is shown when the add preset toolbar button is pressed
    Rectangle {
        id: addPresetCircleDone

        width: units.gu(12)
        height: units.gu(12)

        anchors.centerIn: parent
        border.color: Constants.normalGrey
        color: "#F8F8F8"
        radius: width / 2
        antialiasing: true
        visible: false

        Label {
            id: addPresetTextDone

            x: parent.width/2 - width/2
            y: parent.height/2 - height/2

            color: Constants.green

            fontSize: "large"
            text: i18n.tr("DONE")
        }
    }

    // Add analog clock texts 0,15,30 and 45. Ready for translation to other languages if necessary
    Label {
        anchors {
            horizontalCenter: clockOuterCircle.horizontalCenter
            bottom: clockOuterCircle.top
            bottomMargin: 20
        }

        fontSize: "medium"
        text: i18n.tr("0")
    }

    Label {
        anchors {
            verticalCenter: clockOuterCircle.verticalCenter
            left: clockOuterCircle.right
            leftMargin: 20
        }

        fontSize: "medium"
        text: i18n.tr("15")
    }

    Label {
        anchors {
            horizontalCenter: clockOuterCircle.horizontalCenter
            top: clockOuterCircle.bottom
            topMargin: 20
        }

        fontSize: "medium"
        text: i18n.tr("30")
    }

    Label {
        anchors {
            verticalCenter: clockOuterCircle.verticalCenter
            right: clockOuterCircle.left
            rightMargin: 20
        }

        fontSize: "medium"
        text: i18n.tr("45")
    }

    // Multi-function button (Preset, Start, Stop, Done)
    Button {
        id: buttonTimer

        width: units.gu(12)
        height: units.gu(5)

        anchors {
            bottom: parent.bottom
            bottomMargin: units.gu(1)
            horizontalCenter: parent.horizontalCenter
        }

        color: Constants.brightWhite
        text: i18n.tr("Presets")

        states:[
            State {
                name: "PRESETS"
                PropertyChanges {
                    target: buttonTimer;
                    color: Constants.brightWhite;
                    text: i18n.tr("Presets")
                }
                PropertyChanges {
                    target: timerHand;
                    colortop: Constants.ubuntuOrange
                }
                PropertyChanges {
                    target: clockOuterCircle;
                    glowColor: Constants.analogClockFaceGlowColor
                }
            },
            State {
                name: "START"
                PropertyChanges {
                    target: buttonTimer;
                    color: Constants.ubuntuOrange;
                    text: i18n.tr("Start")
                }
                PropertyChanges {
                    target: timerHand;
                    colortop: Constants.ubuntuOrange
                }
                PropertyChanges {
                    target: clockOuterCircle;
                    glowColor: Constants.analogClockFaceGlowColor
                }
            },
            State {
                name: "STOP"
                PropertyChanges {
                    target: buttonTimer;
                    color: Constants.darkGrey;
                    text: i18n.tr("Stop")
                }
                PropertyChanges {
                    target: timerHand;
                    colortop: Constants.ubuntuOrange
                }
                PropertyChanges {
                    target: clockOuterCircle;
                    glowColor: Constants.analogClockFaceGlowColor
                }
            },
            State {
                name: "DONE"
                PropertyChanges {
                    target: buttonTimer;
                    color: Constants.green;
                    text: i18n.tr("Done")
                }
                PropertyChanges {
                    target: timerHand;
                    colortop: Constants.green
                }
                PropertyChanges {
                    target: clockOuterCircle;
                    glowColor: Constants.analogClockFaceGlowColorTimerDown
                }
            }
        ]

        onClicked: {
            switch (text) {
            case "Presets":
                // PopupUtils.open(setupTimer, buttonTimer)
                state = "PRESETS";
                break;
            case "Start":
                TScript.prepareStartTimer();
                if (totalTime > 0) {
                    timerPage.timerOn = true;
                    state = "STOP";
                }
                break;
            case "Stop":
                timerPage.timerOn = false;
                TScript.resetAll();
                state = "PRESETS";
                break;
            case "Done":
                TScript.resetAll();
                state = "PRESETS";
                break;
            }
        }
    }

    tools: ToolbarActions {
        Action {
            objectName: "add-preset"

            iconSource: Qt.resolvedUrl("../images/add.png")
            text: i18n.tr("Add Preset")

            onTriggered: {
                console.log("You clicked me")
                addPresetCircleDone.visible = true
            }

        }

        Action {
            objectName: "action"

            iconSource: Qt.resolvedUrl("../images/add.png")
            text: i18n.tr("Edit")

            onTriggered: {
                console.log("You clicked me")
                addPresetCircleDone.visible = false
            }
        }
    }
}
