/*
 * Copyright (C) 2013 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Alessandro Pozzi <signor.hyde@gmail.com>
 */

import QtQuick 2.0
import QtQuick.LocalStorage 2.0
import "../common/ClockUtils.js" as Utils

QtObject {
    id: timerStorage

    property var db: null

    Component.onCompleted: {
        Utils.log("TimerStorage loaded")
        openDB();
    }

    // Opens the local db if not already open. On the first call creates the tables
    function openDB() {
        if(db !== null) return;

        db = LocalStorage.openDatabaseSync("ubuntu-clock-app", "0.1", "Default Ubuntu touch clock", 1000);

        try {
            db.transaction(function(tx){
                tx.executeSql('CREATE TABLE IF NOT EXISTS Timer(label TEXT, seconds INTEGER)');
            });
        } catch (err) {
            Utils.error("Error creating table in db");
        }
    }

    function getTimerValue() {
        openDB();

        var result = new Array();
        try {
            db.readTransaction(
                        function(tx){
                            var res = tx.executeSql('SELECT * FROM Timer');
                            if (res.rows.length > 0) {
                                for (var i = 0; i < res.rows.length; i++) {
                                    result[i] = new Array();
                                    result[i][0] = res.rows.item(i).label;
                                    result[i][1] = res.rows.item(i).seconds;
                                }
                            }
                        });
        } catch (err) {
            Utils.error("Error getting timer: " + err);
        }
        return result;
    }

    function getTimerLabel() {
        openDB();

        var result = new Array();
        try {
            db.readTransaction(
                        function(tx){
                            var res = tx.executeSql('SELECT label FROM Timer');
                            if (res.rows.length > 0) {
                                for (var i = 0; i < res.rows.length; i++) {
                                    result[i] = res.rows.item(i).label;
                                }
                            }
                        });
        } catch (err) {
            Utils.error("Error getting timer: " + err);
        }
        return result;
    }

    function getValue(labeltxt) {
        openDB();

        var result;
        try {
            db.readTransaction(
                        function(tx){
                            var res = tx.executeSql('SELECT seconds FROM Timer WHERE label=?', labeltxt);
                            if (res.rows.length > 0) {
                                for (var i = 0; i < res.rows.length; i++) {
                                    result = res.rows.item(i).seconds;
                                    Utils.log("Campo: " + res.rows.item(i).seconds);
                                }
                            }
                        });
        } catch (err) {
            Utils.error("Error getting timer: " + err);
        }
        return result;
    }

    function setTimerValue(labeltxt, secondsint) {
        openDB();

        try {
            db.transaction(function(tx){
                var currentValue;
                var res = tx.executeSql('SELECT seconds FROM Timer WHERE label=?', labeltxt);
                if (res.rows.length > 0) {
                    currentValue = res.rows.item(0).seconds;
                }

                if (currentValue !== secondsint) {
                    // Update existing value or insert if none
                    if (currentValue !== undefined) {
                        res = tx.executeSql('UPDATE Timer SET seconds = ? WHERE label = ?', [secondsint, labeltxt]);
                    } else {
                        res = tx.executeSql('INSERT INTO Timer VALUES(?, ?)', [labeltxt, secondsint]);
                    }
                }
            });
        } catch (err) {
            Utils.error("Error setting labeltxt '"+ labeltxt + "': " + err);
        }
    }

    function deleteTable(table) {
        openDB();

        try {
            db.transaction(
                        function(tx){
                            tx.executeSql("DROP TABLE " + table);
                        });
        } catch (err) {
            Utils.error("Error delete table '" + table + "': " + err);
        }
    }
}
