/*
 * Copyright (C) 2013 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Juha Ristolainen <juha.ristolainen@codemancers.fi>
 *              Nekhelesh Ramananthan <krnekhelesh@gmail.com>
 *              Nick Leppänen Larsson <frals@frals.se>
 *              Marco Biscaro <marcobiscaro2112@gmail.com>
 *              Alessandro Pozzi <signor.hyde@gmail.com>
 */

import QtQuick 2.0
import Ubuntu.Components 0.1
import "common/ClockUtils.js" as Utils
import "common/Constants.js" as Constants
import "common"
import "clock"
import "alarm"
import "stopwatch"
import "timer"

/*!
    \brief MainView with Tabs element.
           First Tab has a single Label and
           second Tab has a single ToolbarAction.
*/

MainView {
    id: root
    // objectName for functional testing purposes (autopilot-qt5)
    objectName: "clock"
    
    ////////////////////////////////////////////////////////////////////////////////
    // Properties of this page
    applicationName: "ubuntu-clock-app"

    width: units.gu(50)
    height: units.gu(75)
    
    ////////////////////////////////////////////////////////////////////////////////
    // New properties of this page
    property var now

    ////////////////////////////////////////////////////////////////////////////////
    // Property changed handlers

    ////////////////////////////////////////////////////////////////////////////////
    // Initialization after page has loaded
    Component.onCompleted: {
        // Any init here
        Utils.log("MainView loaded");
    }

    Component.onDestruction: {
        Utils.log("Closing MainView")

        // Save state
        var lastPage = rootTabs.selectedTabIndex;
        Utils.log("Store lastPage: " + lastPage);
        storage.setSettingsValue(Constants.settingsLastPage, lastPage);
    }

    ////////////////////////////////////////////////////////////////////////////////
    // Helper-functions

    ////////////////////////////////////////////////////////////////////////////////
    // Non-UI elements
    Storage {
        id: storage
        Component.onCompleted: {
            // Set app state from previous run
            var lastPage = storage.getSettingsValue(Constants.settingsLastPage, 0);
            Utils.log("Restore lastPage: " + lastPage);
            rootTabs.selectedTabIndex = lastPage;
        }
    }

    // Timer that runs our app
    Timer {
        id: updateTimer
        objectName: "updateTimer"
        interval: 1000
        repeat: true
        running: true
        onTriggered: {
            now = new Date();
            // FIXME: only send callback to pages that need it
            /* TODO: this timer should probably be changed to fire
              every minute (max) when the app is in the background,
              or just be activated when the app is visible in the
              switcher
              */
            clockPage.onTimerUpdate(now);
            alarmPage.onTimerUpdate(now);
            timerPage.onTimerUpdate(now);
            stopwatchPage.onTimerUpdate(now);
        }
    }

    ////////////////////////////////////////////////////////////////////////////////
    // UI

    // Applications pages in tabs
    Tabs {
        id: rootTabs
        anchors.fill: parent
        
        // First tab begins here
        Tab {
            objectName: "ClockTab"
            
            title: i18n.tr("Clock")
            
            // Tab content begins here
            page: ClockPage {
                id: clockPage
            }
        }
        
        // Second tab begins here
        Tab {
            objectName: "AlarmTab"
            
            title: i18n.tr("Alarm")

            page: AlarmPage {
                id: alarmPage
            }
        }

        Tab {
            objectName: "TimerTab"

            title: i18n.tr("Timer")

            page: TimerPage {
                id: timerPage
            }
        }

        Tab {
            objectName: "StopwatchTab"

            title: i18n.tr("Stopwatch")

            page: StopwatchPage {
                id: stopwatchPage
            }
        }
    }
}
